function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var CheminParams = {
  number: number,
  interger: interger,
  string: string,
  constant: constant,
  optional: optional,
  optionalConst: optionalConst,
  multiple: multiple
};

function string(name) {
  return {
    name: name,
    match: function match(value) {
      if (typeof value === 'string' && value.length > 0) {
        for (var _len = arguments.length, rest = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          rest[_key - 1] = arguments[_key];
        }

        return {
          match: true,
          value: value,
          next: rest
        };
      }

      return {
        match: false
      };
    },
    serialize: function serialize(value) {
      return value.toString();
    },
    stringify: function stringify() {
      return ":" + name;
    }
  };
}

function number(name) {
  return {
    name: name,
    match: function match(value) {
      var parsed = parseFloat(value);

      if (Number.isNaN(parsed)) {
        return {
          match: false
        };
      }

      for (var _len2 = arguments.length, rest = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
        rest[_key2 - 1] = arguments[_key2];
      }

      return {
        match: true,
        value: parsed,
        next: rest
      };
    },
    serialize: function serialize(value) {
      return value.toString();
    },
    stringify: function stringify() {
      return ":" + name + "(number)";
    }
  };
}

function interger(name) {
  return {
    name: name,
    match: function match(value) {
      var parsed = parseInt(value, 10);

      if (Number.isNaN(parsed)) {
        return {
          match: false
        };
      }

      for (var _len3 = arguments.length, rest = new Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
        rest[_key3 - 1] = arguments[_key3];
      }

      return {
        match: true,
        value: parsed,
        next: rest
      };
    },
    serialize: function serialize(value) {
      return value.toString();
    },
    stringify: function stringify() {
      return ":" + name + "(number)";
    }
  };
}

function constant(name) {
  return {
    name: name,
    noValue: true,
    match: function match(value) {
      if (name === value) {
        for (var _len4 = arguments.length, rest = new Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
          rest[_key4 - 1] = arguments[_key4];
        }

        return {
          match: true,
          next: rest,
          value: null
        };
      }

      return {
        match: false
      };
    },
    serialize: function serialize() {
      return name;
    },
    stringify: function stringify() {
      return name;
    }
  };
}

function optional(sub) {
  return {
    name: sub.name,
    match: function match() {
      for (var _len5 = arguments.length, all = new Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
        all[_key5] = arguments[_key5];
      }

      var subMatch = sub.match.apply(sub, all);

      if (subMatch.match) {
        return {
          match: true,
          value: {
            present: true,
            value: subMatch.value
          },
          next: subMatch.next
        };
      }

      return {
        match: true,
        value: {
          present: false
        },
        next: all
      };
    },
    serialize: function serialize(value) {
      return value.present ? sub.serialize(value.value) : null;
    },
    stringify: function stringify() {
      return sub.stringify() + "?";
    }
  };
}

function optionalConst(name, constant) {
  if (constant === void 0) {
    constant = name;
  }

  return {
    name: name,
    match: function match() {
      for (var _len6 = arguments.length, all = new Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
        all[_key6] = arguments[_key6];
      }

      if (all[0] === constant) {
        return {
          match: true,
          value: true,
          next: all.slice(1)
        };
      }

      return {
        match: true,
        value: false,
        next: all
      };
    },
    serialize: function serialize(value) {
      return value ? constant : null;
    },
    stringify: function stringify() {
      return constant + "?";
    }
  };
}

function multiple(sub, atLeastOne) {
  if (atLeastOne === void 0) {
    atLeastOne = false;
  }

  return {
    name: sub.name,
    match: function match() {
      var values = [];

      for (var _len7 = arguments.length, all = new Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
        all[_key7] = arguments[_key7];
      }

      var next = all;
      var nextMatch;

      do {
        nextMatch = sub.match.apply(sub, next);

        if (nextMatch.match) {
          next = nextMatch.next;
          values.push(nextMatch.value);
        }
      } while (nextMatch.match === true);

      if (values.length === 0 && atLeastOne === true) {
        return {
          match: false
        };
      }

      return {
        match: true,
        value: values,
        next: next
      };
    },
    serialize: function serialize(value) {
      return value.map(function (v) {
        return sub.serialize(v);
      }).join('/');
    },
    stringify: function stringify() {
      return "" + sub.stringify() + (atLeastOne ? '+' : '*');
    }
  };
}

var CheminUtils = {
  splitPathname: splitPathname
};

function splitPathname(pathname) {
  var strParts = pathname.split('/');

  if (strParts[0] === '') {
    strParts.shift();
  }

  if (strParts[strParts.length - 1] === '') {
    strParts.pop();
  }

  return strParts;
}

var Chemin = {
  create: createPattern,
  parse: parsePattern,
  serialize: serializePattern,
  stringify: stringifyPattern,
  is: isPattern,
  match: matchPattern,
  matchExact: matchExactPattern
};
var IS_PATTERN =
/*#__PURE__*/
Symbol('IS_PATTERN');

function isPattern(maybe) {
  return maybe && maybe[IS_PATTERN];
}

function createPattern() {
  var _ref;

  for (var _len = arguments.length, parts = new Array(_len), _key = 0; _key < _len; _key++) {
    parts[_key] = arguments[_key];
  }

  return _ref = {
    parts: parts.map(function (part) {
      if (typeof part === 'string') {
        return CheminParams.constant(part);
      }

      return part;
    })
  }, _ref[IS_PATTERN] = true, _ref;
}

function parsePattern(str) {
  var strParts = CheminUtils.splitPathname(str);
  var parts = strParts.map(function (strPart) {
    var isParam = strPart[0] === ':';
    var isOptional = strPart[strPart.length - 1] === '?';
    var name = strPart.slice(isParam ? 1 : 0, isOptional ? -1 : undefined);

    if (isParam === false && isOptional) {
      return CheminParams.optionalConst(name);
    }

    var inner = isParam ? CheminParams.string(name) : CheminParams.constant(name);
    return isOptional ? CheminParams.optional(inner) : inner;
  });
  return createPattern.apply(void 0, parts);
}

function matchPattern(pattern, pathname) {
  var pathParts = typeof pathname === 'string' ? CheminUtils.splitPathname(pathname) : pathname;
  return matchPart(pattern, pathParts);
}

function matchExactPattern(pattern, pathname) {
  var match = matchPattern(pattern, pathname);

  if (match && match.rest.length === 0) {
    return match.params;
  }

  return false;
}

function matchPart(part, pathname) {
  var _params;

  if (isPattern(part)) {
    var match = matchNextParts(part.parts, pathname);

    if (match === false) {
      return false;
    }

    return {
      rest: match.rest,
      params: match.params
    };
  }

  var res = part.match.apply(part, pathname);

  if (res.match === false) {
    return false;
  }

  return {
    params: (_params = {}, _params[part.name] = res.value, _params),
    rest: res.next
  };
}

function matchNextParts(parts, pathname) {
  if (parts.length === 0) {
    return {
      params: {},
      rest: pathname
    };
  }

  var nextPart = parts[0];
  var nextHasParams = isPattern(nextPart) ? true : nextPart.noValue !== true;
  var res = matchPart(nextPart, pathname);

  if (res === false) {
    return false;
  }

  var nextRes = matchNextParts(parts.slice(1), res.rest);

  if (nextRes === false) {
    return false;
  }

  return {
    rest: nextRes.rest,
    params: _extends({}, nextHasParams ? res.params : {}, {}, nextRes.params)
  };
}

function notNull(val) {
  return val !== null;
}

function serializePattern(pattern, params) {
  if (params === void 0) {
    params = {};
  }

  var result = pattern.parts.map(function (part) {
    if (isPattern(part)) {
      return serializePattern(part, params);
    }

    var value = params[part.name];
    return part.serialize(value);
  }).filter(notNull).join('/');

  if (result[0] !== '/') {
    return '/' + result;
  }

  return result;
}

function stringifyPattern(pattern) {
  return '/' + pattern.parts.map(function (part) {
    if (isPattern(part)) {
      return stringifyPattern(part).slice(1);
    }

    return part.stringify();
  }).join('/');
}

export { Chemin, CheminParams, CheminUtils };
//# sourceMappingURL=chemin.esm.js.map
