declare const CheminParams: {
    number: typeof number;
    interger: typeof interger;
    string: typeof string;
    constant: typeof constant;
    optional: typeof optional;
    optionalConst: typeof optionalConst;
    multiple: typeof multiple;
};
declare type PartMatchResult<T> = {
    match: false;
} | {
    match: true;
    value: T extends void ? null : T;
    next: Array<string>;
};
declare type PartMatch<T> = (...parts: Array<string>) => PartMatchResult<T>;
declare type PartSerialize<T> = (value: T) => string | null;
declare type PartStringify = () => string;
declare type CheminParams<N extends string, T> = {
    name: N;
    match: PartMatch<T>;
    stringify: PartStringify;
    serialize: PartSerialize<T>;
} & (T extends void ? {
    noValue: true;
} : {});
declare function string<N extends string>(name: N): CheminParams<N, string>;
declare function number<N extends string>(name: N): CheminParams<N, number>;
declare function interger<N extends string>(name: N): CheminParams<N, number>;
declare function constant<N extends string>(name: N): CheminParams<N, void>;
declare type OptionalValue<T> = {
    present: false;
} | {
    present: true;
    value: T;
};
declare function optional<N extends string, T extends any>(sub: CheminParams<N, T>): CheminParams<N, OptionalValue<T>>;
declare function optionalConst<N extends string>(name: N, constant?: string): CheminParams<N, boolean>;
declare function multiple<N extends string, T extends any>(sub: CheminParams<N, T>, atLeastOne?: boolean): CheminParams<N, Array<T>>;
declare const CheminUtils: {
    splitPathname: typeof splitPathname;
};
declare function splitPathname(pathname: string): Array<string>;
declare const Chemin: {
    create: typeof createPattern;
    parse: typeof parsePattern;
    serialize: typeof serializePattern;
    stringify: typeof stringifyPattern;
    is: typeof isPattern;
    match: typeof matchPattern;
    matchExact: typeof matchExactPattern;
};
declare type Part = CheminParams<any, any> | Chemin<any>;
declare type Params<T> = T extends string ? {} : T extends Chemin<infer P> ? P : T extends CheminParams<infer N, infer P> ? {
    [K in N]: P;
} : {};
declare const IS_PATTERN: unique symbol;
interface Chemin<Params = any> {
    parts: Array<Part>;
    [IS_PATTERN]: Params;
}
declare type In = string | CheminParams<any, any> | Chemin<any>;
declare function isPattern(maybe: any): maybe is Chemin<any>;
declare type Pat<P> = Chemin<P>;
/**
 const r = num=>Array(num).fill(null).map((v,i)=>i);
 const res = r(10).map(count=> count === 0 ? (`function createPattern(): Pat<{}>;`) : (`function createPattern<${r(count).map(i=>`P${i} extends In`).join(', ')}>(${r(count).map(i=>`p${i}: P${i}`).join(', ')}): Pat<${r(count).map(i=>`Params<P${i}>`).join(' & ')}>;`)).map(v=>`// prettier-ignore\n${v}`).join('\n');

 */
declare function createPattern(): Pat<{}>;
declare function createPattern<P0 extends In>(p0: P0): Pat<Params<P0>>;
declare function createPattern<P0 extends In, P1 extends In>(p0: P0, p1: P1): Pat<Params<P0> & Params<P1>>;
declare function createPattern<P0 extends In, P1 extends In, P2 extends In>(p0: P0, p1: P1, p2: P2): Pat<Params<P0> & Params<P1> & Params<P2>>;
declare function createPattern<P0 extends In, P1 extends In, P2 extends In, P3 extends In>(p0: P0, p1: P1, p2: P2, p3: P3): Pat<Params<P0> & Params<P1> & Params<P2> & Params<P3>>;
declare function createPattern<P0 extends In, P1 extends In, P2 extends In, P3 extends In, P4 extends In>(p0: P0, p1: P1, p2: P2, p3: P3, p4: P4): Pat<Params<P0> & Params<P1> & Params<P2> & Params<P3> & Params<P4>>;
declare function createPattern<P0 extends In, P1 extends In, P2 extends In, P3 extends In, P4 extends In, P5 extends In>(p0: P0, p1: P1, p2: P2, p3: P3, p4: P4, p5: P5): Pat<Params<P0> & Params<P1> & Params<P2> & Params<P3> & Params<P4> & Params<P5>>;
declare function createPattern<P0 extends In, P1 extends In, P2 extends In, P3 extends In, P4 extends In, P5 extends In, P6 extends In>(p0: P0, p1: P1, p2: P2, p3: P3, p4: P4, p5: P5, p6: P6): Pat<Params<P0> & Params<P1> & Params<P2> & Params<P3> & Params<P4> & Params<P5> & Params<P6>>;
declare function createPattern<P0 extends In, P1 extends In, P2 extends In, P3 extends In, P4 extends In, P5 extends In, P6 extends In, P7 extends In>(p0: P0, p1: P1, p2: P2, p3: P3, p4: P4, p5: P5, p6: P6, p7: P7): Pat<Params<P0> & Params<P1> & Params<P2> & Params<P3> & Params<P4> & Params<P5> & Params<P6> & Params<P7>>;
declare function createPattern<P0 extends In, P1 extends In, P2 extends In, P3 extends In, P4 extends In, P5 extends In, P6 extends In, P7 extends In, P8 extends In>(p0: P0, p1: P1, p2: P2, p3: P3, p4: P4, p5: P5, p6: P6, p7: P7, p8: P8): Pat<Params<P0> & Params<P1> & Params<P2> & Params<P3> & Params<P4> & Params<P5> & Params<P6> & Params<P7> & Params<P8>>;
declare function createPattern(...parts: Array<In>): Chemin<any>;
declare function parsePattern<Params extends {
    [key: string]: string;
} = {
    [key: string]: string;
}>(str: string): Chemin<Params>;
interface MatchResult<Params> {
    params: Params;
    rest: Array<string>;
}
declare type CheminMatchResult<Params> = MatchResult<Params> | false;
declare function matchPattern<Params>(pattern: Chemin<Params>, pathname: string | Array<string>): CheminMatchResult<Params>;
declare function matchExactPattern<Params>(pattern: Chemin<Params>, pathname: string | Array<string>): false | Params;
declare function serializePattern(pattern: Chemin<{}>): string;
declare function serializePattern<Params>(pattern: Chemin<Params>, params: Params): string;
declare function stringifyPattern(pattern: Chemin<any>): string;
export { Chemin, CheminMatchResult, CheminParams, CheminUtils };
